package com.muyu.common.cache.abs;

import com.muyu.common.cache.HashCache;
import com.muyu.common.redis.service.RedisService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * hash缓存抽象类 HashCacheAbs
 *
 * @author CHX
 * on 2024/3/31 0031
 */
public abstract class HashCacheAbs<K, HK, HV> implements HashCache<K, HK, HV> {

    @Autowired
    private RedisService redisService;

    @Override
    public String encode(K key) {
        return keyPre() + key;
    }

    @Override
    public String encodeHashKey(HK hashKey) {
        return hashKey.toString();
    }

    @Override
    public boolean hashKey(K key,HK hashKey){
        return redisService.hashKey(encode(key),encodeHashKey(hashKey));
    }

    /**
     * 通过Key获取所有的map
     *
     * @param key 数据库键
     * @return 所有集合map
     */
    @Override
    public Map<HK, HV> get(K key) {
        Map<String, HV> cacheMap = redisService.getCacheMap(encode(key));
        if (cacheMap.isEmpty() || cacheMap == null) {
            Map<HK, HV> data = getData(key);
            if (data != null && !data.isEmpty()) {
                cacheMap = encodeMap(data);
            } else {
                cacheMap = encodeMap(defaultValue());
            }
            redisService.setCacheMap(encode(key), cacheMap);
        }
        return decodeMap(cacheMap);
    }

    /**
     * 通过键和hashKey获取数据库hashValue
     *
     * @param key     键
     * @param hashKey hash键
     * @return hash值
     */
    @Override
    public HV get(K key, HK hashKey) {
        HV hashValue = redisService.getCacheMapValue(encode(key), encodeHashKey(hashKey));
        if (hashValue == null){
            HV dataValue = getData(key, hashKey);
            hashValue = dataValue != null ? dataValue : defaultHashValue();
            put(key, hashKey, hashValue);
        }
        return hashValue;
    }

    /**
     * 通过键和hashKey获取数据库hashValue
     *
     * @param key         键
     * @param hashKeyList hash键集合
     * @return hash值
     */
    @Override
    public List<HV> get(K key, HK... hashKeyList) {
        List<String> encodeHashKeyList = Arrays.stream(hashKeyList).map(this::encodeHashKey).toList();
        return redisService.getMultiCacheMapValue(encode(key), encodeHashKeyList);
    }

    /**
     * 获取hash值集合
     *
     * @param key 键
     * @return hash值集合
     */
    @Override
    public List<HV> getToList(K key) {
        Map<HK, HV> hkhvMap = get(key);
        return hkhvMap.values().stream().toList();
    }

    /**
     * 存储数据
     *
     * @param key redis键
     * @param map hashMap集合
     */
    @Override
    public void put(K key, Map<HK, HV> map) {
        redisService.setCacheMap(encode(key), encodeMap(map));
    }

    /**
     * 存储数据
     *
     * @param key      redis键
     * @param dataList 数据值
     * @param hashKey  hash值
     */
    @Override
    public void put(K key, List<HV> dataList, Function<HV, HK> hashKey) {
        HashMap<HK, HV> hashMap = new HashMap<>();
        dataList.forEach((data) -> hashMap.put(hashKey.apply(data), data));
        redisService.setCacheMap(encode(key), encodeMap(hashMap));
    }

    /**
     * 存储数据
     *
     * @param key       redis键
     * @param hashKey   hash键
     * @param hashValue hash值
     */
    @Override
    public void put(K key, HK hashKey, HV hashValue) {
        redisService.setCacheMapValue(encode(key), encodeHashKey(hashKey), hashValue);
    }

    /**
     * 通过redis键删除
     *
     * @param key redis键
     */
    @Override
    public void remove(K key) {
        redisService.deleteObject(encode(key));
    }

    /**
     * 通过redis键和hash键删除
     *
     * @param key     redis键
     * @param hashKey hash键
     */
    @Override
    public void remove(K key, HK hashKey) {
        redisService.deleteCacheMapValue(encode(key), encodeHashKey(hashKey));
    }

    /**
     * 原始数据转编码数据
     *
     * @param dataMap 原始数据
     * @return 编码数据
     */
    private Map<String, HV> encodeMap(Map<HK, HV> dataMap) {
        HashMap<String, HV> encodeDataMap = new HashMap<>();
        dataMap.forEach((hashKey, hashValue) -> encodeDataMap.put(encodeHashKey(hashKey), hashValue));
        return encodeDataMap;
    }

    /**
     * 编码数据转原始数据
     *
     * @param encodeDataMap 编码数据
     * @return 原始数据
     */
    private Map<HK, HV> decodeMap(Map<String, HV> encodeDataMap) {
        HashMap<HK, HV> dataMap = new HashMap<>();
        encodeDataMap.forEach((hashKey, hashValue) -> dataMap.put(decodeHashKey(hashKey), hashValue));
        return dataMap;
    }

    /**
     * 通过键获取所有的hash数据
     *
     * @param key 键
     * @return
     */
    public abstract Map<HK, HV> getData(K key);

    /**
     * 通过缓存键和hash键获取hash值
     *
     * @param key     缓存键
     * @param hashKey hash值
     * @return hash值
     */
    public abstract HV getData(K key, HK hashKey);

    /**
     * 默认值1
     *
     * @return Map<HK, HV>
     */
    public abstract Map<HK, HV> defaultValue();

    /**
     * 默认值2
     *
     * @return HV
     */
    public abstract HV defaultHashValue();
}
