package com.muyu.product.service.impl;

import java.util.ArrayList;
import java.util.List;

import com.muyu.common.core.utils.ObjUtils;
import com.muyu.common.security.utils.SecurityUtils;
import com.muyu.product.domain.*;
import com.muyu.product.domain.model.*;
import com.muyu.product.domain.req.ProjectInfoEditReq;
import com.muyu.product.domain.req.ProjectInfoSaveReq;
import com.muyu.product.domain.resp.CategoryCommonElementResp;
import com.muyu.product.domain.resp.ProjectDetailResp;
import com.muyu.product.domain.resp.ProjectInfoEditResp;
import com.muyu.product.service.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.muyu.product.mapper.ProjectInfoMapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

/**
 * 商品信息Service业务层处理
 *
 * @author DongZeLiang
 * @date 2024-02-27
 */
@Slf4j
@Service
public class ProjectInfoServiceImpl extends ServiceImpl<ProjectInfoMapper, ProjectInfo>  implements ProjectInfoService {

    @Autowired
    private AsProductAttributeInfoService asProductAttributeInfoService;

    @Autowired
    private ProjectSkuInfoService projectSkuInfoService;

    @Autowired
    private AsBrandProjectService asBrandProjectService;

    @Autowired
    private BrandInfoService brandInfoService;

    @Autowired
    private CategoryInfoService categoryInfoService;

    @Autowired
    private RuleAttrInfoService ruleAttrInfoService;

    @Autowired
    private AttributeInfoService attributeInfoService;

    /**
     * 查询商品信息列表
     * @param projectInfo 商品信息
     * @return 商品信息
     */
    @Override
    public List<ProjectInfo> list(ProjectInfo projectInfo) {
        LambdaQueryWrapper<ProjectInfo> queryWrapper = new LambdaQueryWrapper<>();


        if (ObjUtils.notNull(projectInfo.getName())){
            queryWrapper.like(ProjectInfo::getName, projectInfo.getName());
        }

        if (ObjUtils.notNull(projectInfo.getIntroduction())){
            queryWrapper.eq(ProjectInfo::getIntroduction, projectInfo.getIntroduction());
        }

        if (ObjUtils.notNull(projectInfo.getMainType())){
            queryWrapper.eq(ProjectInfo::getMainType, projectInfo.getMainType());
        }

        if (ObjUtils.notNull(projectInfo.getParentType())){
            queryWrapper.eq(ProjectInfo::getParentType, projectInfo.getParentType());
        }

        if (ObjUtils.notNull(projectInfo.getType())){
            queryWrapper.eq(ProjectInfo::getType, projectInfo.getType());
        }

        if (ObjUtils.notNull(projectInfo.getImage())){
            queryWrapper.eq(ProjectInfo::getImage, projectInfo.getImage());
        }

        if (ObjUtils.notNull(projectInfo.getCarouselImages())){
            queryWrapper.eq(ProjectInfo::getCarouselImages, projectInfo.getCarouselImages());
        }

        if (ObjUtils.notNull(projectInfo.getStatus())){
            queryWrapper.eq(ProjectInfo::getStatus, projectInfo.getStatus());
        }

        if (ObjUtils.notNull(projectInfo.getRuleId())){
            queryWrapper.eq(ProjectInfo::getRuleId, projectInfo.getRuleId());
        }

        if (ObjUtils.notNull(projectInfo.getBrandId())){
            queryWrapper.eq(ProjectInfo::getBrandId, projectInfo.getBrandId());
        }

        return list(queryWrapper);
    }

    @Override
    public boolean save(ProjectInfoSaveReq projectInfoSaveReq) {
        ProjectAddModel projectAddModel = projectInfoSaveReq.getProjectAddModel();
        ProjectInfo projectInfo = ProjectInfo.saveModelBuild(projectAddModel, SecurityUtils::getUsername);
        boolean save = this.save(projectInfo);
        if(save){
            // 商品属性
            List<AttrValueModel> attrValueList = projectInfoSaveReq.getAttrValueList();
            List<AsProductAttributeInfo> asProductAttributeInfoList = attrValueList.stream()
                    .map(attrValueModel -> AsProductAttributeInfo.attrValueModelBuild(attrValueModel, projectInfo::getId))
                    .toList();
            if(!asProductAttributeInfoList.isEmpty()){
                asProductAttributeInfoService.saveBatch(asProductAttributeInfoList);
            }

            // 商品sku
            List<ProjectSkuModel> projectSkuList = projectInfoSaveReq.getProjectSkuList();
            List<ProjectSkuInfo> projectSkuInfoList = projectSkuList.stream()
                    .map(projectSkuModel -> ProjectSkuInfo.projectSkuModelBuild(projectSkuModel, projectInfo::getId))
                    .toList();
            if(!projectSkuInfoList.isEmpty()){
                projectSkuInfoService.saveBatch(projectSkuInfoList);
            }
        }
        return save;
    }


    @Override
    public ProjectInfoEditResp getDetailById(Long id) {
        // 获取商品基本信息
        ProjectInfo projectInfo = this.getById(id);

        // 商品属性
        LambdaQueryWrapper<AsProductAttributeInfo> asProductAttributeInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        asProductAttributeInfoLambdaQueryWrapper.eq(AsProductAttributeInfo::getProductId, id);
        List<AsProductAttributeInfo> asProductAttributeInfoList = asProductAttributeInfoService.list(asProductAttributeInfoLambdaQueryWrapper);

        // 商品规格信息
        LambdaQueryWrapper<ProjectSkuInfo> projectSkuInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
        projectSkuInfoLambdaQueryWrapper.eq(ProjectSkuInfo::getProjectId, id);
        List<ProjectSkuInfo> projectSkuInfoList = projectSkuInfoService.list(projectSkuInfoLambdaQueryWrapper);

        ProjectInfoEditResp projectInfoEditResp = ProjectInfoEditResp.allBuild(projectInfo, asProductAttributeInfoList, projectSkuInfoList);

        return projectInfoEditResp;
    }

    @Override
    public boolean updateById(Long id, ProjectInfoEditReq projectInfoEditReq) {
        ProjectAddModel projectAddModel = projectInfoEditReq.getProjectAddModel();

        // 删除品牌中间表
        LambdaQueryWrapper<AsBrandProject> asBrandProjectLambdaQueryWrapper = new LambdaQueryWrapper<>();
        asBrandProjectLambdaQueryWrapper.eq(AsBrandProject::getProjectId,id);
        asBrandProjectService.remove(asBrandProjectLambdaQueryWrapper);

        asBrandProjectService.save(AsBrandProject.builder()
                .brandId(projectAddModel.getBrandId())
                .projectId(String.valueOf(id))
                .build()
        );

        ProjectInfo projectInfo = ProjectInfo.editBuild(id, projectAddModel);
        boolean update = this.updateById(projectInfo);
        if(update){
            //删除属性值
            LambdaQueryWrapper<AsProductAttributeInfo> asProductAttributeInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
            asProductAttributeInfoLambdaQueryWrapper.eq(AsProductAttributeInfo::getProductId,id);
            asProductAttributeInfoService.remove(asProductAttributeInfoLambdaQueryWrapper);

            //添加属性值
            List<AttrValueModel> attrValueList = projectInfoEditReq.getAttrValueList();
            List<AsProductAttributeInfo> asProductAttributeInfoList = attrValueList.stream()
                    .map(attrValueModel -> AsProductAttributeInfo.attrValueModelBuild(attrValueModel, projectInfo::getId))
                    .toList();
            if(!asProductAttributeInfoList.isEmpty()){
                asProductAttributeInfoService.saveBatch(asProductAttributeInfoList);
            }

            // 删除sku
            LambdaQueryWrapper<ProjectSkuInfo> projectSkuInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
            projectSkuInfoLambdaQueryWrapper.eq(ProjectSkuInfo::getProjectId,id);
            projectSkuInfoService.remove(projectSkuInfoLambdaQueryWrapper);

            //添加sku
            List<ProjectSkuModel> projectSkuList = projectInfoEditReq.getProjectSkuList();
            List<ProjectSkuInfo> projectSkuInfoList = projectSkuList.stream()
                    .map(projectSkuModel -> ProjectSkuInfo.projectSkuModelBuild(projectSkuModel, projectInfo::getId))
                    .toList();
            if(!projectSkuInfoList.isEmpty()){
                projectSkuInfoService.saveBatch(projectSkuInfoList);
            }
        }

        return update;

//        // 删除商品属性
//        LambdaQueryWrapper<AsProductAttributeInfo> asProductAttributeInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
//        asProductAttributeInfoLambdaQueryWrapper.eq(AsProductAttributeInfo::getProductId,id);
//        asProductAttributeInfoService.remove(asProductAttributeInfoLambdaQueryWrapper);
//
//        // 删除商品sku
//        LambdaQueryWrapper<ProjectSkuInfo> projectSkuInfoLambdaQueryWrapper = new LambdaQueryWrapper<>();
//        projectSkuInfoLambdaQueryWrapper.eq(ProjectSkuInfo::getProjectId,id);
//        projectSkuInfoService.remove(projectSkuInfoLambdaQueryWrapper);
//
//        // 修改商品基本属性
//        ProjectEditModel projectEditModel = projectInfoEditReq.getProjectEditModel();
//        ProjectInfo projectInfo = ProjectInfo.editBuild(id, projectEditModel);
//        boolean update = this.updateById(projectInfo);
//        if(update){
//            // 添加商品属性
//            List<AttrValueModel> attrValueList = projectInfoEditReq.getAttrValueList();
//            List<AsProductAttributeInfo> asProductAttributeInfoList = attrValueList.stream()
//                    .map(attrValueModel -> AsProductAttributeInfo.attrValueModelBuild(attrValueModel,projectInfo::getId))
//                    .toList();
//            if(!asProductAttributeInfoList.isEmpty()){
//                asProductAttributeInfoService.saveBatch(asProductAttributeInfoList);
//            }
//
//            // 添加商品sku
//            List<ProjectSkuModel> projectSkuList = projectInfoEditReq.getProjectSkuList();
//            List<ProjectSkuInfo> projectSkuInfoList = projectSkuList.stream()
//                    .map(projectSkuModel -> ProjectSkuInfo.projectSkuModelBuild(projectSkuModel, projectInfo::getId))
//                    .toList();
//            if(!projectSkuInfoList.isEmpty()){
//                projectSkuInfoService.saveBatch(projectSkuInfoList);
//            }
//        }
//        return update;
    }


    @Override
    public ProjectDetailResp getDetailInfo(Long id) {
        // 商品信息获取
        ProjectInfo projectInfo = this.getById(id);

        // 品牌信息
        BrandInfo brandInfo = this.brandInfoService.getById(projectInfo.getBrandId());


        // 品类集合
        List<CategoryInfo> categoryInfoList = this.categoryInfoService.listByIds(new ArrayList<>() {{
            add(projectInfo.getMainType());
            add(projectInfo.getParentType());
            add(projectInfo.getType());
        }});

        // 商品sku集合
        List<ProjectSkuInfo> projectSkuInfoList = this.projectSkuInfoService.list(new LambdaQueryWrapper<>() {{
            eq(ProjectSkuInfo::getProjectId, id);
        }});

        // 商品和属性集合
        List<AsProductAttributeInfo> productAttributeInfoList = this.asProductAttributeInfoService.list(new LambdaQueryWrapper<>() {{
            eq(AsProductAttributeInfo::getProductId, id);
        }});

        // 商品规格
        List<RuleAttrAddModel> ruleAttrModelList = this.ruleAttrInfoService.list(new LambdaQueryWrapper<>() {{
                    eq(RuleAttrInfo::getRuleId, projectInfo.getRuleId());
                }}).stream()
                .map(RuleAttrAddModel::infoBuild).toList();

        // 属性集合 属性组集合
        CategoryCommonElementResp templateAttribute = this.categoryInfoService.
                getTemplateAttributeByCateGoryId(projectInfo.getType());

        List<TemplateAttributeGroupModel> templateAttributeGroupList = templateAttribute.getTemplateAttributeGroupList();
        ArrayList<TemplateAttributeModel> templateAttributeList = new ArrayList<>() {{
            addAll(templateAttribute.getTemplateAttributeList());
        }};

        // 商品属性组和商品属性的id
        ArrayList<Long> notInAttributeIdList = new ArrayList<>();
        List<Long> attributeGroupIdList = templateAttributeGroupList.stream()
                .flatMap(templateAttributeGroupModel -> templateAttributeGroupModel.getAttributeList().stream())
                .map(TemplateAttributeModel::getId)
                .toList();
        if(!attributeGroupIdList.isEmpty()){
            notInAttributeIdList.addAll(attributeGroupIdList);
        }
        List<Long> attributeIdList = templateAttributeList.stream()
                .map(TemplateAttributeModel::getId)
                .toList();
        if(!attributeIdList.isEmpty()){
            notInAttributeIdList.addAll(attributeIdList);
        }

        // 添加商品的自由属性
        List<AsProductAttributeInfo> productAttributeList = this.asProductAttributeInfoService.list(new LambdaQueryWrapper<>() {{
            eq(AsProductAttributeInfo::getProductId, projectInfo.getId());
            notIn(AsProductAttributeInfo::getAttributeId, notInAttributeIdList);
        }});

        List<TemplateAttributeModel> projectAttributeList = new ArrayList<>();
        if(!productAttributeList.isEmpty()){
            List<Long> attrIdList = productAttributeList.stream()
                    .map(AsProductAttributeInfo::getAttributeId)
                    .toList();
            projectAttributeList = attributeInfoService.list(
                    new LambdaQueryWrapper<>(){{
                        in(AttributeInfo::getId,attrIdList);
                    }}
            ).stream()
                    .map(TemplateAttributeModel::attributeInfoBuild)
                    .toList();
        }

        // 把自由属性添加到商品属性的集合中
        if(!projectAttributeList.isEmpty()){
            templateAttributeList.addAll(projectAttributeList);
        }

        return ProjectDetailResp.builder()
                .projectInfo(projectInfo)
                .brandInfo(brandInfo)
                .categoryInfoList(categoryInfoList)
                .projectSkuInfoList(projectSkuInfoList)
                .productAttributeInfoList(productAttributeInfoList)
                .ruleAttrModelList(ruleAttrModelList)
                .attributeInfoList(templateAttributeList)
                .attributeGroupList(templateAttributeGroupList)
                .build();
    }
}
